import React, { useState, useEffect } from 'react';
import { Space, Table, Button, Badge, Divider, Input } from 'antd';
import { useSetState } from 'ahooks';
import type { TableProps, ColumnsType, ColumnType } from 'antd/es/table';
import { t } from '@/utils/i18n';
import { PlusOutlined, ExclamationCircleFilled, SearchOutlined } from '@ant-design/icons';
import { antdUtils } from '@/utils/antd';
import { useRequest } from '@/hooks/use-request';
import type { DictTypeVO, DictDataVO, DictDataPageReqVO } from '@/models'
import dictDataService from '@/request/service/system-dictdata';
import DictDataEditor from './dict-data-editor';

type DataIndex = keyof DictDataVO;

interface DictDataProps {
    data?: DictTypeVO | null;
}

export default (props: DictDataProps) => {
    const { data } = props;
    const showDeleteConfirm = (item: DictDataVO) => {
        antdUtils.modal?.confirm({
            title: `确认删除值为 ${item.value} 的字典吗?`,
            icon: <ExclamationCircleFilled />,
            content: '请注意删除以后不可恢复！',
            okText: '删除',
            okType: 'danger',
            cancelText: '取消',
            onOk() {
                return new Promise(async (resolve) => {
                    const [error, { code, msg }] = await deleteDictDataApi(item.id);
                    if (error || code !== 0) {
                        antdUtils.message?.open({ type: 'error', content: msg ?? '操作失败' })
                    } else {
                        antdUtils.message?.open({ type: 'success', content: '删除成功' })
                    }
                    await load();
                    resolve('')
                }).catch(() => antdUtils.message?.open({
                    type: 'error',
                    content: '操作失败',
                }));
            },
            onCancel() {
            },
        });
    };

    const [editorVisable, seEditorVisable] = useState<boolean>(false);
    const [editData, seEditData] = useState<DictDataVO>();
    const [dataSource, setDataSource] = useState<DictDataVO[]>([]);
    const [total, setTotal] = useState(0);
    const [searchState, setSearchState] = useSetState<DictDataPageReqVO>({ dictType: data?.type ?? "" });
    const [onSearching, setOnSearching] = useState(false);

    const { runAsync: getDictDataPageApi } = useRequest(dictDataService.getDictDataPageApi, { manual: true });
    const { runAsync: deleteDictDataApi } = useRequest(dictDataService.deleteDictDataApi, { manual: true });

    const showEditor = (record: DictDataVO | undefined) => {
        seEditData(record);
        seEditorVisable(true);
    }

    const load = async () => {
        setOnSearching(true);
        const [error, { data }] = await getDictDataPageApi(searchState);
        setOnSearching(false);
        if (!error) {
            setDataSource(data.list);
            setTotal(data.total);
        }
    };

    const getColumnSearchProps = (dataIndex: DataIndex): ColumnType<DictDataVO> => ({
        filterDropdown: ({ setSelectedKeys, selectedKeys, confirm, clearFilters, close }) => (
            <div style={{ padding: 8 }} onKeyDown={(e) => e.stopPropagation()}>
                <Input.Search
                    placeholder='输入要搜索的标签'
                    value={selectedKeys[0]}
                    onChange={(e) => {
                        setSelectedKeys(e.target.value && e.target.value !== '' ? [e.target.value] : [])
                    }}
                    onSearch={(value) => {
                        if (value === '' && clearFilters) {
                            clearFilters!!()
                        }
                        confirm();
                    }}
                    onPressEnter={() => confirm()}
                    allowClear
                    style={{ marginBottom: 8, display: 'block' }}
                    enterButton="搜索"
                    size="middle"
                    loading={onSearching}
                />
            </div>
        ),
        filterIcon: (filtered: boolean) => (
            <SearchOutlined style={{ color: filtered ? 'primaryColor' : undefined }} />
        ),
    });

    const columns: ColumnsType<DictDataVO> = [
        {
            title: '数据标签',
            dataIndex: 'label',
            key: 'label',
            align: 'center',
            width: 150,
            ...getColumnSearchProps('label')
        },
        {
            title: '数据键值',
            dataIndex: 'value',
            key: 'value',
            align: 'center',
            width: 120,
        },

        {
            title: '状态',
            key: 'status',
            dataIndex: 'status',
            align: 'center',
            width: 120,
            render: (value: number) => {
                return (value === 0 ? <Badge status="success" text="已开启" /> : <Badge status="error" text="已关闭" />)
            }
        },
        {
            title: t("QkOmYwne" /* 操作 */),
            key: 'action',
            align: 'center',
            render: (_, record) => (
                <Space size="middle" split={(
                    <Divider type='vertical' />
                )}>
                    <a
                        onClick={() => showEditor(record)}>
                        编辑
                    </a>
                    <a
                        onClick={() => {
                            showDeleteConfirm(record)
                        }}>
                        删除
                    </a>
                </Space>
            ),
            width: 150,
        },
    ];

    useEffect(() => {
        load();
    }, [searchState]);

    useEffect(() => {
        if (props.data)
            setSearchState({ dictType: props.data!!.type })
    }, [props])

    const onChange: TableProps<DictDataVO>['onChange'] = (pagination, filters, sorter, extra) => {
        const state: DictDataPageReqVO = {
            label: filters.label ? filters.label[0] as string : undefined,
            dictType: props.data?.type ?? "",
            pageNo: pagination.current ?? 1,
            pageSize: pagination.pageSize
        }
        setSearchState(state);
    };

    return (
        <>
            <div>
                {data ? (<div className="py-[8px] flex flex-row-reverse">
                    <Button className="ml-5" type='primary' size='middle' icon={<PlusOutlined />}
                        onClick={() => showEditor(undefined)}> 添加字典数据 </Button>
                </div>) : null}

                <Table rowKey="id"
                    scroll={{ x: true }}
                    columns={columns}
                    dataSource={dataSource}
                    className='bg-transparent'
                    onChange={onChange}
                    pagination={{
                        pageSize: searchState.pageSize,
                        current: searchState.pageNo,
                        total,
                        position: ['bottomRight']
                    }}
                />
            </div>
            <DictDataEditor onSave={() => {
                load();
                seEditorVisable(false);
            }}
                onCancel={() => { seEditorVisable(false) }}
                visible={editorVisable}
                data={editData}
                dictType={data?.type ?? ""} />
        </>

    );
};
