import React, { useState, useEffect, useMemo } from 'react';
import { Space, Table, Button, Badge, Divider, Input } from 'antd';
import { useSetState } from 'ahooks';
import type { TableProps, ColumnsType, ColumnType } from 'antd/es/table';
import { t } from '@/utils/i18n';
import { PlusOutlined, ExclamationCircleFilled, SearchOutlined } from '@ant-design/icons';
import { antdUtils } from '@/utils/antd';
import { useRequest } from '@/hooks/use-request';
import type { DictTypeVO, DictTypePageReqVO } from '@/models'
import dictTypeService from '@/request/service/system-dicttype';
import DictTypeEditor from './dict-type-editor';

type DataIndex = keyof DictTypeVO;

interface DictTypeProps {
    onSelectDictType?: (data: DictTypeVO) => void;
}

export default (props: DictTypeProps) => {
    const showDeleteConfirm = (item: DictTypeVO) => {
        antdUtils.modal?.confirm({
            title: `确认删除名称为: "${item.name}" 的字典吗?`,
            icon: <ExclamationCircleFilled />,
            content: '请注意删除以后不可恢复！',
            okText: '删除',
            okType: 'danger',
            cancelText: '取消',
            onOk() {
                return new Promise(async (resolve, reject) => {
                    const [error, { code, msg }] = await deleteDictTypeApi(item.id);
                    if (error || code !== 0) {
                        antdUtils.message?.open({ type: 'error', content: msg ?? '操作失败' })
                    } else {
                        antdUtils.message?.open({ type: 'success', content: '删除成功' })
                    }
                    await load();
                    resolve('');
                }).catch(() => antdUtils.message?.open({
                    type: 'error',
                    content: '操作失败',
                }));
            },
            onCancel() {
            },
        });
    };

    const [editorVisable, seEditorVisable] = useState<boolean>(false);
    const [editData, seEditData] = useState<DictTypeVO>();
    const [dataSource, setDataSource] = useState<DictTypeVO[]>([]);
    const [total, setTotal] = useState(0);
    const [searchState, setSearchState] = useSetState<DictTypePageReqVO>({});
    const [onSearching, setOnSearching] = useState(false);

    const { runAsync: getDictTypePageApi } = useRequest(dictTypeService.getDictTypePageApi, { manual: true });
    const { runAsync: deleteDictTypeApi } = useRequest(dictTypeService.deleteDictTypeApi, { manual: true });

    const showEditor = (record: DictTypeVO | undefined)=> {
        seEditData(record);
        seEditorVisable(true);
    }

    const load = async () => {
        const [error, { data, code, msg }] = await getDictTypePageApi(searchState);
        setOnSearching(false);
        if (!error) {
            setDataSource(data.list);
            setTotal(data.total);
        } else {
            if (error || code !== 0) {
                antdUtils.message?.open({ type: 'error', content: msg ?? '操作失败' });
                return
            }
        }
    };

    const getColumnSearchProps = (dataIndex: DataIndex): ColumnType<DictTypeVO> => ({
        filterDropdown: ({ setSelectedKeys, selectedKeys, confirm, clearFilters, close }) => (
            <div style={{ padding: 8 }} onKeyDown={(e) => e.stopPropagation()}>
                <Input.Search
                    placeholder={dataIndex === 'name' ? '输入要搜索的字典名称' : '输入要搜索的字典类型'}
                    value={selectedKeys[0]}
                    onChange={(e) => {
                        setSelectedKeys(e.target.value && e.target.value !== '' ? [e.target.value] : [])
                    }}
                    onSearch={(value) => {
                        if (value === '' && clearFilters) {
                            clearFilters!!()
                        }
                        confirm();
                    }}
                    onPressEnter={() => confirm()}
                    allowClear
                    style={{ marginBottom: 8, display: 'block' }}
                    enterButton="搜索"
                    size="middle"
                    loading={onSearching}
                />
            </div>
        ),
        filterIcon: (filtered: boolean) => (
            <SearchOutlined style={{ color: filtered ? 'primaryColor' : undefined }} />
        ),
    });

    const columns: ColumnsType<DictTypeVO> = [
        {
            title: '字典名称',
            dataIndex: 'name',
            key: 'name',
            align: 'center',
            width: 150,
            ...getColumnSearchProps('name')
        },
        {
            title: '字典类型',
            dataIndex: 'type',
            key: 'type',
            align: 'center',
            width: 120,
            ...getColumnSearchProps('type')
        },

        {
            title: '状态',
            key: 'status',
            dataIndex: 'status',
            align: 'center',
            width: 120,
            render: (value: number) => {
                return (value === 0 ? <Badge status="success" text="已开启" /> : <Badge status="error" text="已关闭" />)
            }
        },
        {
            title: t("QkOmYwne" /* 操作 */),
            key: 'action',
            align: 'center',
            render: (_, record) => (
                <Space size="middle" split={(
                    <Divider type='vertical' />
                )}>
                    <a onClick={() => showEditor(record) }>
                        编辑
                    </a>
                    <a
                        onClick={() => {
                            showDeleteConfirm(record)
                        }}>
                        删除
                    </a>
                </Space>
            ),
            width: 150,
        },
    ];

    useEffect(() => {
        load();
    }, [searchState]);

    useMemo(() => {
        console.log('onMemChanged: ' + editData)
    }, [editData])

    const onChange: TableProps<DictTypeVO>['onChange'] = (pagination, filters, sorter, extra) => {
        const state: DictTypePageReqVO = {
            name: filters.name ? filters.name[0] as string : undefined,
            type: filters.type ? filters.type[0] as string : undefined,
            pageNo: pagination.current,
            pageSize: pagination.pageSize
        }
        setSearchState(state);
    };

    return (
        <>
            <div>
                <div className="py-[8px] flex flex-row-reverse">
                    <Button className="ml-5" 
                        type='primary' size='middle' icon={<PlusOutlined />}
                        onClick={() => showEditor(undefined)}> 新增字典 </Button>
                </div>
                <Table rowKey="id"
                    scroll={{ x: true }}
                    onChange={onChange}
                    columns={columns}
                    dataSource={dataSource}
                    rowSelection={{
                        type: 'radio',
                        onSelect: (record, select) => {
                            if (select && props.onSelectDictType)
                                props.onSelectDictType!!(record)
                        }
                    }}
                    className='bg-transparent'
                    pagination={{
                        pageSize: searchState.pageSize,
                        current: searchState.pageNo,
                        total,
                        position: ['bottomRight']
                    }}
                />
            </div>
            <DictTypeEditor onSave={() => {
                    load();
                    seEditorVisable(false);
                }}
                onCancel={() => { seEditorVisable(false) }}
                visible={editorVisable}
                data={editData}/>
        </>
    );
};
